@@ -51,6 +51,13 @@ export const ClipboardHistory = {
 	KeepAll: 2,
 };
 
+export const DefaultColors = {
+	'custom-bg-color': ['rgb(54,54,58)', 'rgb(250,250,251)'],
+	'custom-fg-color': ['rgb(255,255,255)', 'rgb(34,34,38)'],
+	'custom-card-bg-color': ['rgb(71,71,76)', 'rgb(255,255,255)'],
+	'custom-search-bg-color': ['rgb(71,71,76)', 'rgb(255,255,255)'],
+};
+
 export const UserAgent = 'Mozilla/5.0 (compatible; CopyousBot/1.0; +https://github.com/boerdereinar/copyous)';
 
 export const HljsCdns = [
