@@ -293,6 +293,7 @@ let SearchEntry = class SearchEntry extends St.Entry {
 		// Item button
 		const left = new St.BoxLayout();
 		this.primary_icon = left;
+		left.connect('key-press-event', (_btn, event) => this._leftKeyPressEvent(event));
 		this._itemButton = new St.Button({
 			style_class: 'search-entry-button item-button',
 			button_mask: St.ButtonMask.ONE | St.ButtonMask.TWO,
@@ -341,6 +342,7 @@ let SearchEntry = class SearchEntry extends St.Entry {
 		// Right
 		const right = new St.BoxLayout();
 		this.secondary_icon = right;
+		right.connect('key-press-event', (_btn, event) => this._rightKeyPressEvent(event));
 
 		// Pin button
 		const pinButton = new St.Button({
@@ -526,22 +528,30 @@ let SearchEntry = class SearchEntry extends St.Entry {
 		return Clutter.EVENT_PROPAGATE;
 	}
 
+	_leftKeyPressEvent(event) {
+		const key = event.get_key_symbol();
+		if (key === Clutter.KEY_Right || key === Clutter.KEY_KP_Right) {
+			this.clutter_text.grab_key_focus();
+			return Clutter.EVENT_STOP;
+		}
+		return Clutter.EVENT_PROPAGATE;
+	}
+
+	_rightKeyPressEvent(event) {
+		const key = event.get_key_symbol();
+		if (key === Clutter.KEY_Left || key === Clutter.KEY_KP_Left) {
+			this.clutter_text.grab_key_focus();
+			return Clutter.EVENT_STOP;
+		}
+		return Clutter.EVENT_PROPAGATE;
+	}
+
 	vfunc_style_changed() {
 		super.vfunc_style_changed();
 		const themeNode = this.get_theme_node();
 		this.natural_width = themeNode.get_max_width();
 	}
 
-	vfunc_navigate_focus(from, direction) {
-		if (direction === St.DirectionType.TAB_BACKWARD) {
-			return Clutter.EVENT_STOP;
-		}
-		if (direction === St.DirectionType.TAB_FORWARD || direction === St.DirectionType.DOWN) {
-			this.emit('navigate-focus-out');
-		}
-		return super.vfunc_navigate_focus(from, direction);
-	}
-
 	vfunc_scroll_event(event) {
 		const direction = event.get_scroll_direction();
 		const swap = this.ext.settings.get_boolean('swap-scroll-shortcut');
@@ -586,11 +596,10 @@ SearchEntry = __decorate(
 				type: GObject.ParamSpec.string('type', null, null, GObject.ParamFlags.READWRITE, ''),
 			},
 			Signals: {
-				'search': {
+				search: {
 					param_types: [SearchQuery.$gtype],
 				},
-				'activate': {},
-				'navigate-focus-out': {},
+				activate: {},
 			},
 		}),
 	],
